@@ -82,6 +82,7 @@ struct _EHTMLEditorViewClass {
 						 GdkEventButton *event);
 	void		(*paste_primary_clipboard)
 						(EHTMLEditorView *view);
+	void		(*is_ready)		(EHTMLEditorView *view);
 };
 
 enum EHTMLEditorViewHistoryEventType {
@@ -207,6 +208,10 @@ gchar *		e_html_editor_view_get_text_html
 						(EHTMLEditorView *view,
 						 const gchar *from_domain,
 						 GList **inline_images);
+gchar *		e_html_editor_view_get_text_html_for_drafts_with_images
+						(EHTMLEditorView *view,
+						 const gchar *from_domain,
+						 GList **inline_images);
 gchar *		e_html_editor_view_get_text_html_for_drafts
 						(EHTMLEditorView *view);
 gchar *		e_html_editor_view_get_body_text_html_for_drafts
@@ -281,6 +286,11 @@ gboolean	e_html_editor_view_is_message_from_edit_as_new
 void		e_html_editor_view_set_is_message_from_edit_as_new
 						(EHTMLEditorView *view,
 						 gboolean value);
+gboolean	e_html_editor_view_content_is_new_message
+						(EHTMLEditorView *view);
+void		e_html_editor_view_set_content_is_new_message
+						(EHTMLEditorView *view,
+						 gboolean value);
 void		e_html_editor_view_insert_quoted_text
 						(EHTMLEditorView *view,
 						 const gchar *text);
@@ -316,6 +326,17 @@ void		e_html_editor_view_remove_input_event_listener_from_body
 						(EHTMLEditorView *view);
 void		e_html_editor_view_register_input_event_listener_on_body
 						(EHTMLEditorView *view);
+gchar *		e_html_editor_view_insert_signature
+						(EHTMLEditorView *view,
+						 const gchar *content,
+						 gboolean is_html,
+						 const gchar *id,
+						 gboolean *set_signature_from_message,
+						 gboolean *check_if_signature_is_changed,
+						 gboolean *ignore_next_signature_change);
+gboolean	e_html_editor_view_is_ready	(EHTMLEditorView *view);
+void		e_html_editor_view_reconnect_paste_clipboard_signals
+						(EHTMLEditorView *view);
 G_END_DECLS
 
 #endif /* E_HTML_EDITOR_VIEW_H */
