@@ -1624,7 +1624,7 @@ static EPopupMenu ethi_context_menu[] = {
 		N_("Sort _Descending"),
 		G_CALLBACK (ethi_popup_sort_descending), 2),
 	E_POPUP_ITEM (
-		N_("_Unsort"), G_CALLBACK (ethi_popup_unsort), 0),
+		N_("_Reset sort"), G_CALLBACK (ethi_popup_unsort), 0),
 	E_POPUP_SEPARATOR,
 	E_POPUP_ITEM (
 		N_("Group By This _Field"),
@@ -2028,8 +2028,10 @@ ethi_event (GnomeCanvasItem *item,
 				ethi->click_x = event_x_win;
 				ethi->click_y = event_y_win;
 				ethi->maybe_drag = TRUE;
+				col = -1;
 				is_pointer_on_division (ethi, x, &start, &col);
-				ethi->selected_col = col;
+				if (col != -1)
+					ethi->selected_col = col;
 				if (gtk_widget_get_can_focus (GTK_WIDGET (item->canvas)))
 					e_canvas_item_grab_focus (item, TRUE);
 			} else if (event_button == 3) {
