@@ -946,6 +946,7 @@ receive_done (gpointer data)
 			local_outbox,
 			CAMEL_TRANSPORT (info->service),
 			E_FILTER_SOURCE_OUTGOING,
+			FALSE,
 			info->cancellable,
 			receive_get_folder, info,
 			receive_status, info,
@@ -960,7 +961,7 @@ receive_done (gpointer data)
 			GTK_PROGRESS_BAR (info->progress_bar), 1.0);
 
 		if (info->state == SEND_CANCELLED)
-			text = _("Canceled");
+			text = _("Cancelled");
 		else {
 			text = _("Complete");
 			info->state = SEND_COMPLETE;
@@ -1291,7 +1292,7 @@ refresh_folders_exec (struct _refresh_folders_msg *m,
 				g_clear_error (&local_error);
 				break;
 			} else if (!g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
-				CamelStore *store = m->store;
+				CamelStore *store;
 				const gchar *full_name;
 
 				if (folder) {
@@ -1610,6 +1611,7 @@ send_receive (GtkWindow *parent,
 				session, local_outbox,
 				CAMEL_TRANSPORT (info->service),
 				E_FILTER_SOURCE_OUTGOING,
+				FALSE,
 				info->cancellable,
 				receive_get_folder, info,
 				receive_status, info,
@@ -1719,6 +1721,7 @@ mail_receive_service (CamelService *service)
 			local_outbox,
 			CAMEL_TRANSPORT (service),
 			E_FILTER_SOURCE_OUTGOING,
+			FALSE,
 			info->cancellable,
 			receive_get_folder, info,
 			receive_status, info,
@@ -1735,8 +1738,9 @@ exit:
 	g_object_unref (session);
 }
 
-void
-mail_send (EMailSession *session)
+static void
+do_mail_send (EMailSession *session,
+	      gboolean immediately)
 {
 	CamelFolder *local_outbox;
 	CamelService *service;
@@ -1792,6 +1796,7 @@ mail_send (EMailSession *session)
 		session, local_outbox,
 		CAMEL_TRANSPORT (service),
 		E_FILTER_SOURCE_OUTGOING,
+		immediately,
 		info->cancellable,
 		receive_get_folder, info,
 		receive_status, info,
@@ -1799,3 +1804,15 @@ mail_send (EMailSession *session)
 
 	g_object_unref (service);
 }
+
+void
+mail_send (EMailSession *session)
+{
+	do_mail_send (session, FALSE);
+}
+
+void
+mail_send_immediately (EMailSession *session)
+{
+	do_mail_send (session, TRUE);
+}
